//-----------------------------------------------------------------------------
// File: Difficulty.h
//
// Desc: Difficulty settings for advisor behaviors
//
// Copyright (c) 2000-2008 Black Element Software. All rights reserved
//-----------------------------------------------------------------------------

//! minimal units required to conquer given island (NULL == attack on Carrier)
int MinAssaultUnits( CCPlayer player, IslandModel island )
{
	int cnt;

	// base solely on behavior for now
	if( player.GetBehavior() == PLAYER_BEHAVIOR_OFFENSIVE )
	{
		cnt = 2;
		if ( !island )
			cnt = 1;
		else
		{
			if( island.GetType() == IslandDefence )
				cnt = 4;
		}
	}
	else
	{
		cnt = 3;
		if( !island )
			cnt = 4;
		else
		{
			if( island.GetType() == IslandDefence )
				cnt = 6;
		}
	}

	return cnt;
}

//! time to conquer one island keypoint in distant simulation
float KeypointCapTime( CCPlayer player, IslandModel island )
{
	//const float TIME_MIN = 50.0; //se skillem 1, bude trvat zabrani ostrov cca 15 minut
	//const float TIME_DIFF = 160.0; //

	float skill = 1.0 - player.GetSkill();
	float ret = skill * TIME_DIFF + TIME_MIN;
	ret *= GetCampaign().GetIslandForceMod( island );
	ret *= 60.0;
	diict = ret;
	return ret;
}
/*
float KeypointCapTime( CCPlayer player, IslandModel island )
{
	const float TIME_MIN = 50.0; //se skillem 1, bude trvat zabrani ostrov cca 15 minut
	const float TIME_DIFF = 160.0; //

	float skill = 1.0 - player.GetSkill();
	float ret = skill * TIME_DIFF + TIME_MIN;
	ret *= GetCampaign().GetIslandForceMod( island );
	ret *= 60.0;
	return ret;
}
*/
//! loss probability for one unit when fighting over a keypoint
//TODO: could damage assemblies to achieve more credible results?
//float LossChance( CCPlayer player, IslandModel island, EntityBasic unitBasic )
//{
//	//TODO: apply armor modifiers
//
//	const float BASE = 0.02;
//	const float DIFF = 0.08;
//
//	float skill = 1.0 - player.GetSkill();
//	float ret = skill * DIFF + BASE;
//	if( island.GetType() == IslandDefence ) return ret * 1.5;
//	return ret;
//}

//! max base damage done to unit during a keypoint fight
float BaseDamage( CCPlayer player, IslandModel island, EntityBasic unitBasic )
{
	const float BASE = 0.0;
	const float DIFF = 0.2;

	float skill = 1.0 - player.GetSkill();
	return skill * DIFF + BASE;
}

//! minimal carrier health that will cause panic (flee)
float MinSafeCarrierHealth( CCPlayer player )
{
	//TODO: dodelat zdravi jednotlivych repairZones
	Carrier carrier = player.GetCarrier();
	if ( carrier )
	{
		InteractiveRepairZone rz =  carrier.GetLocalRepairZone( ECARRIER_ZONE_ENGINES );
		
		if ( rz.GetHealth() <= 0.8 )
			return 1;

		rz = carrier.GetLocalRepairZone( ECARRIER_ZONE_DEFENSE );

		if ( rz.GetHealth() <= 0.05 )
			return 1;
	}

	if ( player.GetSkill() >= 0.5 )
		return 0.6;
	else
		return 0.4;
}

//! time between AI attacks
float PeaceTime( CCPlayer player )
{
	// lower skill AI will attack with larger gaps
	const float BASE = 10.0;
	const float DIFF = 70.0;

	//TEMP: for testing
	//return 0;
	if( player == GetGame().GetControlledPlayer() )
		return 0;

	float skill = 1.0 - player.GetSkill();
	float ret = skill * DIFF + BASE;
	if ( player.GetBehavior() == PLAYER_BEHAVIOR_DEFENSIVE )
		ret *= 1.5;

	if ( g_GM && g_Campaign.CurrentChapter > 2 )
		ret = 10;

	ret *= 60;

	return ret;
}

//! timeout for "forgetting" a defeat by island
float IslandDefeatTimeout( CCPlayer player )
{
	// lower skill AI will attack with larger gaps
	const float BASE = 50.0;
	const float DIFF = 150.0;

	if( player == GetGame().GetControlledPlayer() ) return 0;

	float skill = 1.0 - player.GetSkill();
	float ret = skill * DIFF + BASE;

	if( player.GetBehavior() == PLAYER_BEHAVIOR_DEFENSIVE ) return ret * 1.5;
	
	ret *= 1000;
	return ret;
}

//! timeout for "forgetting" a defeat by enemy carrier
float CarrierDefeatTimeout( CCPlayer player )
{
	// lower skill AI will attack with larger gaps
	const float BASE = 50.0;
	const float DIFF = 150.0;

	if( player == GetGame().GetControlledPlayer() ) return 0;

	float skill = 1.0 - player.GetSkill();
	float ret = skill * DIFF + BASE;

	if( player.GetBehavior() == PLAYER_BEHAVIOR_DEFENSIVE ) return ret * 1.5;

	ret *= 1000;
	return ret;
}

//! importance modifier of an island being considered for assault
float CapImportance( IslandModel island, CCPlayer player )
{
	int id = island.GetArea();
	int whatWouldEnemyLoose = g_Game.GetIslandImportance( id );
	int whatWouldIGain = g_Game.GetIslandCapImportance( id, player );

	const float NOT_IMPORTANT = 0.1;
	float res = 0;

	if( player.GetBehavior() == PLAYER_BEHAVIOR_DEFENSIVE )
	{
		// try to keep my network working
		res = whatWouldIGain;
		res += whatWouldEnemyLoose * NOT_IMPORTANT;
	}
	else
	{
		// try to infiltrate enemy network
		res = whatWouldEnemyLoose;
		res += whatWouldIGain * NOT_IMPORTANT;
	}

	return res;
}