// File: TeleControl.h
//
// Desc: Carrier Command: TeleControl solver (for AI/Player/Planning switching, etc.)
//
// Author: raist
//
// Copyright (c) 2000-2011 Black Element Software. All rights reserved
//-----------------------------------------------------------------------------



class TeleControlSpider extends CCTeleControl
{
	
	void TeleControlSpider()
	{
	}

	void ~TeleControlSpider()
	{
	}

	void OnEnterUnit( InteractiveEntity pEntity )
	{

		// in map, always visible!
		if( IsInMap() )
		{
			ContextMenu(true);
			return;
		}

		// carrier has never menu visible unless not in map
		if( IsInherited(pEntity,Type("Carrier")) )
		{
			ContextMenu(false);
			return;
		}

		// only carrier unit comes to indirect mode
		if( pEntity.IsCarrierUnit() )
		{
			// if not drive mode... let remain autopilot
			if( !pEntity.IsDriveMode() )
				return;
		}
		else
		{
			// this is for normal map units
			pEntity.AutoPilot(false);
			ContextMenu(false);
			return;
		}
		
		// are we running AI ?
		if( IsInAction() )
		{
			if( pEntity.HasPlan()  )
			{
				if( !pEntity.IsPlanSuspended() )
					ContextMenu(true);
			}
			else
			{
				ContextMenu(false);
				pEntity.AutoPilot(false);
			}
		}
		else
		{
			// show always in map
			ContextMenu(true);
		}

		
	}
	void OnLeaveUnit( InteractiveEntity pEntity )
	{
		if ( !di_no_auto_resume )
		{
		if( pEntity.HasPlan() )
		{
			if( !pEntity.IsPlanSuspended() || HasDirectControl() )
				pEntity.PlanEnable();
		}

		pEntity.AutoPilot(true);
		}
		if ( di_no_auto_resume )
		{
		if( pEntity.HasPlan() )
		{
			if( !pEntity.IsPlanSuspended() )
			{
				pEntity.PlanEnable();
				pEntity.AutoPilot(true);
			}
			else
			{
			if( pEntity.IsPlanSuspended() )
				pEntity.AutoPilot(false);
			}
		}
		else
			pEntity.AutoPilot(true)
		}
	}

	void OnContextMenuHideAttempt( InteractiveEntity pEntity )
	{
		if( pEntity )
		{
			if( pEntity.HasPlan() && !pEntity.IsPlanSuspended() )
			{
				ContextMenu(true);
				return;
			}
		}

		ContextMenu(false);
	}

	void OnMap( InteractiveEntity pEntity )
	{
		//player is in Map
		if( IsInMap() )
		{
			Action();

			if( pEntity )
			{
				if( !pEntity.HasPlan() || pEntity.IsPlanSuspended() )
				{
					pEntity.AutoPilot(false);
					ContextMenu(false);
				}
				else
				{
					pEntity.AutoPilot(true);
					ContextMenu(true);
				}
			}
		}
		
		//player is not in Map (3D view)
		if( IsInAction() )
		{
			if( pEntity )
			{
				if( pEntity.IsCarrierUnit() || !pEntity.IsIndoor() )
				{
					if( CanSwitchToMap() ) 
					{
						Map();
						ContextMenu(true);
						pEntity.ForceStopFire();
					}
				}
			}
		}
	}

	void OnPlanCompleted( InteractiveEntity pEntity )
	{
		if( !HasDirectControl() )
		{
			pEntity.AutoPilot(true);
			ContextMenu(true);
		}
	}

	void OnPlanUndock( InteractiveEntity pEntity )
	{
		if( HasDirectControl() )
		{
			pEntity.AutoPilot(false);
		}
		else
		{
			if( pEntity.IsSelected() && !pEntity.HasPlan() )
			{
				pEntity.AutoPilot(false);
				pEntity.ProcessUndock();
				return;
			}
			
			pEntity.AutoPilot(true);
			pEntity.ProcessUndock();

			if( pEntity.HasPlan() )
				pEntity.PlanEnable();
		}
	}
	void OnPlanDock( InteractiveEntity pEntity )
	{
		if( pEntity.HasPlan() )
		{
			pEntity.PlanClear();
		}

		pEntity.ProcessDock();
	}

	void OnUnDock( InteractiveEntity pEntity )
	{
		if( !HasDirectControl() )
			pEntity.AutoPilot(true);
	}
	void OnDock( InteractiveEntity pEntity )
	{
		pEntity.AutoPilot(false);
	}
	
	int DetermineMove( InteractiveEntity pEntity, InteractiveEntity pControlled )
	{
		float fManta = 0, fWalrus = 0, fCharacter = 0;
		float distance = pEntity.GetDistanceTo(pControlled);
		
		// if Walrus - select only Walruses 
		if( pEntity.GetAssembly().IsWalrus() )
		{
			if( pControlled.GetAssembly().IsManta() )
				return 0;
		}

		if( pEntity.IsWingman() )
		{
			return 1;
		}

		// in this case, use only wingmans
		if( pControlled.GetPlayer().HasWingmans() )
			return 0;

		if( pControlled.GetAssembly().IsManta() )
		{
			if( pEntity.GetAssembly().IsWalrus() )
				return 0;

			fManta = 500;
			fWalrus = 0;
			fCharacter = 0;
		}
		
		if( pControlled.GetAssembly().IsWalrus() )
		{
			fManta = 0;
			fWalrus = 250;
			fCharacter = 0;
		}

		// to units
		fManta = fManta * 32;
		fWalrus = fWalrus * 32;
		fCharacter = fCharacter * 32;

		if( pEntity.GetAssembly().IsManta() )
		{
			if( distance < fManta && distance > 0 )
				return 1;
		}
		
		if( pEntity.GetAssembly().IsWalrus() )
		{
			if( distance < fWalrus && distance > 0 )
				return 1;
		}

		// not suitable
		return 0;
	}

	int DetermineAttack( InteractiveEntity pEntity, InteractiveEntity pControlled )
	{
		float fManta = 0, fWalrus = 0, fCharacter = 0;
		float distance = pEntity.GetDistanceTo(pControlled);
		
		// if Walrus - select only Walruses 
		if( pEntity.GetAssembly().IsWalrus() )
		{
			if( pControlled.GetAssembly().IsManta() )
				return 0;
		}

		if( pEntity.IsWingman() )
		{
			return 1;
		}

		// in this case, use only wingmans
		if( pControlled.GetPlayer().HasWingmans() )
			return 0;

		if( pControlled.GetAssembly().IsManta() )
		{
			if( pEntity.GetAssembly().IsWalrus() )
				return 0;

			fManta = 500;
			fWalrus = 0;
			fCharacter = 0;
		}
		
		if( pControlled.GetAssembly().IsWalrus() )
		{
			fManta = 0;
			fWalrus = 250;
			fCharacter = 0;
		}

		// to units
		fManta = fManta * 32;
		fWalrus = fWalrus * 32;
		fCharacter = fCharacter * 32;

		if( pEntity.GetAssembly().IsManta() )
		{
			if( distance < fManta && distance > 0 )
				return 1;
		}
		
		if( pEntity.GetAssembly().IsWalrus() )
		{
			if( distance < fWalrus && distance > 0 )
				return 1;
		}

		// not suitable
		return 0;
	}

	//! Determine if suitable for Assist action
	int DetermineAssist( InteractiveEntity pEntity, InteractiveEntity pControlled )
	{
		float fManta = 0, fWalrus = 0, fCharacter = 0;
		float distance = pEntity.GetDistanceTo(pControlled);

		// walrus cannot assist to manta
		if( pEntity.GetAssembly().IsWalrus() )
		{
			if( pControlled.GetAssembly().IsManta() )
				return 0;
		}

		if( pEntity.IsWingman() )
		{
			return 1;
		}

		// in this case, use only wingmans
		if( pControlled.GetPlayer().HasWingmans() )
			return 0;

		if( pEntity.IsDocking() )
			return 0;		

		if( pControlled.GetAssembly().IsManta() )
		{
			// walrus cannot assist to manta
			if( pEntity.GetAssembly().IsWalrus() )
				return 0;

			fManta = 500;
			fWalrus = 0;
			fCharacter = 0;
		}
		
		if( pControlled.GetAssembly().IsWalrus() )
		{
			fManta = 0;
			fWalrus = 250;
			fCharacter = 0;
		}

		// to units
		fManta = fManta * 32;
		fWalrus = fWalrus * 32;
		fCharacter = fCharacter * 32;

		if( pEntity.GetAssembly().IsManta() )
		{
			if( distance < fManta && distance > 0 )
				return 1;
		}
		
		if( pEntity.GetAssembly().IsWalrus() )
		{
			if( distance < fWalrus && distance > 0 )
				return 1;
		}

		// not suitable
		return 0;
	}
	//! Determine if suitable for Emergency action
	int DetermineEmergency( InteractiveEntity pEntity, InteractiveEntity pControlled )
	{
		float fManta = 0, fWalrus = 0, fCharacter = 0;
		float distance = pEntity.GetDistanceTo(pControlled);

		if( pEntity.IsDocking() )
		{
			return 0;	
		}

		if( pEntity.IsWingman() )
		{
			return 1;
		}

		if( pControlled.GetAssembly().IsManta() )
		{
			fManta = 500;
			fWalrus = 200;
			fCharacter = 0;
		}
		
		if( pControlled.GetAssembly().IsWalrus() )
		{
			fManta = 1000;
			fWalrus = 350;
			fCharacter = 0;
		}

		// to units
		fManta = fManta * 32;
		fWalrus = fWalrus * 32;
		fCharacter = fCharacter * 32;

		if( pEntity.GetAssembly().IsManta() )
		{
			if( distance < fManta && distance > 0 )
				return 1;
		}
		
		if( pEntity.GetAssembly().IsWalrus() )
		{
			if( distance < fWalrus && distance > 0 )
				return 1;
		}

		// not suitable
		return 0;
	}
	//! Determine if suitable for Hold action
	int DetermineHold( InteractiveEntity pEntity, InteractiveEntity pControlled )
	{

		if( !pEntity.HasPlan() )
			return 0;

		if( pEntity.IsPlanSuspended() )
			return 0;

		// prefer wigmans
		if( pEntity.IsWingman() )
			return 1;

		// in this case, use only wingmans
		if( pControlled.GetPlayer().HasWingmans() )
			return 0;
	
		// suitable
		return 1;
	}
	//! Determine if suitable for Cancel action
	int DetermineContinue( InteractiveEntity pEntity, InteractiveEntity pControlled )
	{

		if( !pEntity.HasPlan() )
			return 0;

		// prefer wigmans
		if( pEntity.IsWingman() )
			return 1;

		// in this case, use only wingmans
		if( pControlled.GetPlayer().HasWingmans() )
			return 0;

		// suitable
		return 1;
	}
	//! Determine if suitable for Offensive mode
	int DetermineOffensive( InteractiveEntity pEntity, InteractiveEntity pControlled )
	{
		if( pEntity )
		{
			if( pEntity.IsWingman() )
				return 1;
		}

		// not suitable
		return 0;
	}
	//! Determine if suitable for Deffensive mode
	int DetermineDefensive( InteractiveEntity pEntity, InteractiveEntity pControlled )
	{
		if( pEntity )
		{
			if( pEntity.IsWingman() )
				return 1;
		}

		// not suitable
		return 0;
	}
	//! Determine if suitable for launch (deploy)
	int DetermineLaunch( InteractiveEntity pEntity, InteractiveEntity pControlled )
	{
		if( pEntity )
		{
			if ( pEntity.GetAssembly().IsManta() != pControlled.GetAssembly().IsManta() && !pEntity.IsWingman() && di_radial_menu_same_type && pControlled.GetAssembly().IsWalrus() != pControlled.GetAssembly().IsManta() )
				return 0;
			if( pEntity.IsDocked() )
				return 1;
		}

		// not suitable
		return 0;
	}
	//! Determine if suitable for dock
	int DetermineDock( InteractiveEntity pEntity, InteractiveEntity pControlled )
	{
		if( pEntity )
		{
			if ( pEntity.GetAssembly().IsManta() != pControlled.GetAssembly().IsManta() && !pEntity.IsWingman() && di_radial_menu_same_type && pControlled.GetAssembly().IsWalrus() != pControlled.GetAssembly().IsManta() )
				return 0;
			if( !pEntity.IsDocked() && !pEntity.IsDocking() )
				return 1;
		}

		// not suitable
		return 0;
	}



	//! Select single unit for proper action
	int OnBasicSelect( InteractiveEntity pEntity, InteractiveEntity pControlled, int eActionType )
	{
		// check if entity not cargo
		if( pEntity )
		{
			if( pEntity.IsCargo() )
				return 0;
		}
	
		// launch
		if( eActionType == ETELEACTION_LAUNCH )
		{
			return DetermineLaunch(pEntity,pControlled);
		}

		// dock
		if( eActionType == ETELEACTION_DOCK )
		{
			return DetermineDock(pEntity,pControlled);
		}

		// move
		if( eActionType == ETELEACTION_MOVE )
		{
			if( !pControlled )
				return 0;
				
			return DetermineMove(pEntity,pControlled);
		}

		// attack
		if( eActionType == ETELEACTION_ATTACK )
		{
			if( !pControlled )
				return 0;

			return DetermineAttack(pEntity,pControlled);
		}

		// assist
		if( eActionType == ETELEACTION_ASSIST )
		{
			if( !pControlled )
				return 0;
					
			return DetermineAssist(pEntity,pControlled);
		}

		// emergency
		if( eActionType == ETELEACTION_EMERGENCY )
		{
			if( !pControlled )
				return 0;
		
			return DetermineEmergency(pEntity,pControlled);
		}

		// offensive stance
		if( eActionType == ETELEACTION_OFFENSIVE )
		{
			return DetermineOffensive(pEntity,pControlled);
		}

		// defensive stance
		if( eActionType == ETELEACTION_DEFENSIVE )
		{
			return DetermineDefensive(pEntity,pControlled);
		}

		// hold action
		if( eActionType == ETELEACTION_HOLD )
		{
			if( !pControlled )
				return 0;
		
			return DetermineHold(pEntity,pControlled);
		}

		// hold action
		if( eActionType == ETELEACTION_CONTINUE )
		{
			if( !pControlled )
				return 0;
		
			return DetermineContinue(pEntity,pControlled);
		}

		// do not select by default
		return 0;
	}


	//! Process single unit for proper action
	void OnBasicProcess( InteractiveEntity pEntity, InteractiveEntity pControlled, int eActionType )
	{
		if( pEntity )
		{
			if( pEntity.IsCargo() )
				return;
		}

		// launch
		if( eActionType == ETELEACTION_LAUNCH )
		{
			return;
		}

		// dock
		if( eActionType == ETELEACTION_DOCK )
		{
			return;
		}

		// move
		if( eActionType == ETELEACTION_MOVE )
		{
			if( pEntity.IsDocked() )
				pEntity.PlanSuspend();

			return;
		}

		// attack
		if( eActionType == ETELEACTION_ATTACK )
		{
			if( pEntity.IsDocked() )
				pEntity.PlanSuspend();

			return;
		}

		// assist
		if( eActionType == ETELEACTION_ASSIST )
		{
			if( !pControlled )
				return;
		}

		// emergency
		if( eActionType == ETELEACTION_EMERGENCY )
		{
			if( !pControlled )
				return;
		}

		// plan suspend
		if( eActionType == ETELEACTION_SUSPEND || eActionType == ETELEACTION_HOLD )
		{
			bool processSuspend = true;
			if( pControlled.GetPlayer().HasWingmans() )
				processSuspend = pEntity.IsWingman();
		
			if( processSuspend )
			{
				if( pEntity.HasPlan() )
					pEntity.PlanSuspend();
			}
		}

	
		// plan enable
		if( eActionType == ETELEACTION_ENABLE || eActionType == ETELEACTION_CONTINUE )
		{
			bool processEnable = true;
			if( pControlled.GetPlayer().HasWingmans() )
				processEnable = pEntity.IsWingman();
			
					
			if( processEnable )
			{
				if( pEntity.HasPlan() )
				{
					if( pEntity.IsPlanSuspended() )
						pEntity.PlanEnable();
				}
				pEntity.AutoPilot(true);
			}
			
		}
		
	}

	//! Init announcement in script
	void InitAnnouncement()
	{
		// initialize message array
		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_CARRIER, "carrier", 0 );
		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_MANTA, "manta", 0 );
		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_WALRUS, "walrus", 0 );
		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_UNITS, "units", 0 );

		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_MANTA2, "manta2", 0 );
		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_MANTA3, "manta3", 0 );
		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_MANTA4, "manta4", 0 );
		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_MANTA5, "manta5", 0 );		

		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_WALRUS6, "walrus6", 0 );
		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_WALRUS7, "walrus7", 0 );
		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_WALRUS8, "walrus8", 0 );
		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_WALRUS9, "walrus9", 0 );

		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_0, "number0", 0 );
		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_1, "number1", 0 );

		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_2, "number2", 0 );
		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_3, "number3", 0 );
		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_4, "number4", 0 );
		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_5, "number5", 0 );

		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_6, "number6", 0 );
		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_7, "number7", 0 );
		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_8, "number8", 0 );
		AddAnnouncement( ERPMSGP_UNIT, ERADIO_UNIT_9, "number9", 0 );

		AddAnnouncement( ERPMSGP_LOW, ERADIO_UNIT_UNDERATTACK, "underattack", 1.2 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_UNIT_ENGAGEDENEMY, "engagedenemy", 1 );
		AddAnnouncement( ERPMSGP_HIGH, ERADIO_UNIT_DESTROYED, "destroyed", 0 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_UNIT_LOWFUEL, "lowfuel", 0 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_UNIT_LOWAMMO, "lowammo", 0 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_UNIT_NOFUEL, "nofuel", 0 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_UNIT_NOAMMO, "noammo", 0 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_UNIT_DOCKED, "docked", 0.5 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_UNIT_LAUNCHED, "launched", 0.5 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_UNIT_LEAVINGTELEMETRY, "leavingtelemetry", 0.5 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_UNIT_OUTOFTELEMETRY, "outoftelemetry", 1 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_UNIT_BACKTOTELEMETRY, "backtotelemetry", 0 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_UNIT_PROGRAMCOMPLETED, "programcompleted", 1 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_UNIT_READY, "ready", 0.5 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_UNIT_ONMOVE, "onmove", 0 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_UNIT_ISSTUCK, "isstuck", 1.5 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_UNIT_ISFUBAR, "isfubar", 0 );
		AddAnnouncement( ERPMSGP_ULTIMATE, ERADIO_UNIT_HEAVILYDAMAGED, "heavilydamaged", 1.0 );
		AddAnnouncement( ERPMSGP_ULTIMATE, ERADIO_UNIT_CANTLANDDUECARGO, "cantlandwithcargo", 0 );
		AddAnnouncement( ERPMSGP_ULTIMATE, ERADIO_UNIT_CANTATTACK, "cantattack", 0 );

		AddAnnouncement( ERPMSGP_REPAIRSYSTEM, ERADIO_CARRIER_SUPERSTRUCTURE, "c_superstructure", 0 );
		AddAnnouncement( ERPMSGP_REPAIRSYSTEM, ERADIO_CARRIER_ENGINES, "c_engines", 0 );
		AddAnnouncement( ERPMSGP_REPAIRSYSTEM, ERADIO_CARRIER_CONTROLTOWER, "c_controltower", 0 );
		AddAnnouncement( ERPMSGP_REPAIRSYSTEM, ERADIO_CARRIER_OFFENSE, "c_offense", 0 );
		AddAnnouncement( ERPMSGP_REPAIRSYSTEM, ERADIO_CARRIER_DEFENSE, "c_defense", 0 );
		AddAnnouncement( ERPMSGP_REPAIRSYSTEM, ERADIO_CARRIER_CRUISEMISSILE, "c_cruisemissile", 0 );
		AddAnnouncement( ERPMSGP_REPAIRSYSTEM, ERADIO_CARRIER_MANTALIFTS, "c_mantalifts", 0 );
		AddAnnouncement( ERPMSGP_REPAIRSYSTEM, ERADIO_CARRIER_WALRUSLEFT, "c_walrusleft", 0 );
		AddAnnouncement( ERPMSGP_REPAIRSYSTEM, ERADIO_CARRIER_WALRUSRIGHT, "c_walrusright", 0 );
		AddAnnouncement( ERPMSGP_REPAIRSYSTEM, ERADIO_CARRIER_REPAIRSYSTEM, "c_repairsystem", 0 );

		AddAnnouncement( ERPMSGP_LOW, ERADIO_CARRIER_UAV, "c_uav", 0 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_CARRIER_BARQUE, "c_barque", 0 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_CARRIER_SAILING, "c_sailing", 1 );

		AddAnnouncement( ERPMSGP_HIGH, ERADIO_CARRIER_INOPERABLE, "c_inoperable", 1.5 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_CARRIER_TAKING_DAMAGE, "c_takingdamage", 0 );
		AddAnnouncement( ERPMSGP_ULTIMATE, ERADIO_CARRIER_HEAVILY_DAMAGED, "c_heavilydamaged", 0 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_CARRIER_REPAIRED, "c_repaired", 0 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_CARRIER_REPAIRING, "c_repairing", 0 );

		AddAnnouncement( ERPMSGP_LOW, ERADIO_STRAT_COURSE_SET, "m_courseset", 0 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_STRAT_STOCKPILESELECTED, "m_stockpile", 0 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_STRAT_DESTINATION_REACHED, "m_destination", 0 );

		// TODO:
		AddAnnouncement( ERPMSGP_LOW, ERADIO_STRAT_DEPLOYDEVICEDESTROYED, "", 1 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_STRAT_CARRIERDEPLOYING, "", 1 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_STRAT_CANTLEAVEVEHICLE_NOSPACE, "", 1 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_STRAT_CANTLEAVEVEHICLE_WATER, "", 1 );
		AddAnnouncement( ERPMSGP_LOW, ERADIO_STRAT_CANTLEAVEVEHICLE_SPEED, "", 1 );
		// TODO:


		AddAnnouncement( ERPMSGP_HIGH, ERADIO_TACTICS_ASSIST, "t_assist", 0 );
		AddAnnouncement( ERPMSGP_HIGH, ERADIO_TACTICS_ATTACK, "t_attack", 0 );
		AddAnnouncement( ERPMSGP_HIGH, ERADIO_TACTICS_DEPLOY_ALL, "t_deployall", 0 );
		AddAnnouncement( ERPMSGP_HIGH, ERADIO_TACTICS_DOCK_ALL, "t_dockall", 0 );
		AddAnnouncement( ERPMSGP_ULTIMATE, ERADIO_TACTICS_EMERGENCY, "t_emergency", 0 );
		AddAnnouncement( ERPMSGP_ULTIMATE, ERADIO_TACTICS_MOVE, "t_move", 0 );
		AddAnnouncement( ERPMSGP_HIGH, ERADIO_TACTICS_OFFENSIVE, "t_offensive", 0);
		AddAnnouncement( ERPMSGP_HIGH, ERADIO_TACTICS_DEFENSIVE, "t_defensive", 0 );
		AddAnnouncement( ERPMSGP_HIGH, ERADIO_TACTICS_HOLD, "", 0 );
		AddAnnouncement( ERPMSGP_HIGH, ERADIO_TACTICS_CONTINUE, "", 0 );
	}

	//! On announcement event create
	void OnAnnouncementCreate( int eMessage, int eUnitMessage )
	{
	}

	//! Compress announcement event
	void OnAnnouncementCompress( int eMessage, int eUnitMessage, int eType )
	{

		if ( eMessage == ERADIO_STRAT_DEPLOYDEVICEDESTROYED )
		{
			if ( eUnitMessage != ERADIO_NONE )
			{
				if ( ContainAnnouncement(eMessage,ERADIO_STRAT_DEPLOYDEVICEDESTROYED) )
					CancelAnnouncement(eMessage,ERADIO_STRAT_DEPLOYDEVICEDESTROYED);
			}
		}

		if ( eMessage == ERADIO_STRAT_CARRIERDEPLOYING )
		{
			if ( eUnitMessage != ERADIO_NONE )
			{
				if ( ContainAnnouncement(eMessage,ERADIO_STRAT_CARRIERDEPLOYING) )
					CancelAnnouncement(eMessage,ERADIO_STRAT_CARRIERDEPLOYING);
			}
		}

		if ( eMessage == ERADIO_STRAT_CANTLEAVEVEHICLE_NOSPACE )
		{
			if ( eUnitMessage != ERADIO_NONE )
			{
				if ( ContainAnnouncement(eMessage,ERADIO_STRAT_CANTLEAVEVEHICLE_NOSPACE) )
					CancelAnnouncement(eMessage,ERADIO_STRAT_CANTLEAVEVEHICLE_NOSPACE);
			}
		}

		if ( eMessage == ERADIO_STRAT_CANTLEAVEVEHICLE_WATER )
		{
			if ( eUnitMessage != ERADIO_NONE )
			{
				if ( ContainAnnouncement(eMessage,ERADIO_STRAT_CANTLEAVEVEHICLE_WATER) )
					CancelAnnouncement(eMessage,ERADIO_STRAT_CANTLEAVEVEHICLE_WATER);
			}
		}

		if ( eMessage == ERADIO_STRAT_CANTLEAVEVEHICLE_SPEED )
		{
			if ( eUnitMessage != ERADIO_NONE )
			{
				if ( ContainAnnouncement(eMessage,ERADIO_STRAT_CANTLEAVEVEHICLE_SPEED) )
					CancelAnnouncement(eMessage,ERADIO_STRAT_CANTLEAVEVEHICLE_SPEED);
			}
		}

		if( eType != ERPMSGP_UNIT )
			return;

		if( eMessage == ERADIO_CARRIER_SAILING )
		{
			if ( ContainAnnouncement(eMessage,ERADIO_CARRIER_SAILING) )
				CancelAnnouncement(eMessage,ERADIO_CARRIER_SAILING);
		}
		
		if( eMessage == ERADIO_UNIT_LAUNCHED )
		{
			int manta = 0;
			int walrus = 0;
			
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA2) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA3) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA4) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA5) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS6) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS7) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS8) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS9) )
				walrus++;

			if( manta > 1 || walrus > 1 )
			{
				CreateMultipleAnnouncement(eMessage,ERADIO_UNIT_LAUNCHED);
			
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA2);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA3);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA4);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA5);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS6);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS7);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS8);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS9);
			}

			return;
		}

		if( eMessage == ERADIO_UNIT_READY )
		{
			manta = 0;
			walrus = 0;
			
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA2) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA3) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA4) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA5) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS6) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS7) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS8) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS9) )
				walrus++;

			if( manta > 1 || walrus > 1 )
			{
				CreateMultipleAnnouncement(eMessage,ERADIO_UNIT_READY);

				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA2);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA3);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA4);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA5);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS6);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS7);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS8);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS9);
			}
			return;
		}

		if( eMessage == ERADIO_UNIT_ENGAGEDENEMY )
		{
			manta = 0;
			walrus = 0;
			
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA2) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA3) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA4) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA5) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS6) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS7) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS8) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS9) )
				walrus++;

			if( manta > 1 || walrus > 1 )
			{
				CreateMultipleAnnouncement(eMessage,ERADIO_UNIT_ENGAGEDENEMY);

				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA2);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA3);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA4);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA5);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS6);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS7);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS8);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS9);
			}
			return;
		}

		if( eMessage == ERADIO_UNIT_DOCKED )
		{
			manta = 0;
			walrus = 0;
			
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA2) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA3) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA4) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA5) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS6) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS7) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS8) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS9) )
				walrus++;

			if( manta > 1 || walrus > 1 )
			{
				CreateMultipleAnnouncement(eMessage,ERADIO_UNIT_DOCKED);

				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA2);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA3);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA4);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA5);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS6);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS7);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS8);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS9);
			}
			return;
		}

		if( eMessage == ERADIO_UNIT_UNDERATTACK )
		{
			manta = 0;
			walrus = 0;
			
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA2) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA3) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA4) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA5) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS6) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS7) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS8) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS9) )
				walrus++;

			if( manta > 1 || walrus > 1 )
			{
				CreateMultipleAnnouncement(eMessage,ERADIO_UNIT_UNDERATTACK);

				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA2);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA3);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA4);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA5);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS6);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS7);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS8);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS9);
			}
			return;
		}

		if( eMessage == ERADIO_UNIT_LEAVINGTELEMETRY )
		{
			manta = 0;
			walrus = 0;
			
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA2) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA3) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA4) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA5) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS6) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS7) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS8) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS9) )
				walrus++;

			if( manta > 1 || walrus > 1 )
			{
				CreateMultipleAnnouncement(eMessage,ERADIO_UNIT_LEAVINGTELEMETRY);

				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA2);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA3);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA4);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA5);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS6);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS7);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS8);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS9);
			}
			return;
		}

		if( eMessage == ERADIO_UNIT_OUTOFTELEMETRY )
		{
			manta = 0;
			walrus = 0;
			
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA2) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA3) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA4) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA5) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS6) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS7) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS8) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS9) )
				walrus++;

			if( manta > 1 || walrus > 1 )
			{
				CreateMultipleAnnouncement(eMessage,ERADIO_UNIT_OUTOFTELEMETRY);

				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA2);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA3);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA4);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA5);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS6);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS7);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS8);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS9);
			}
			return;
		}

		if( eMessage == ERADIO_UNIT_PROGRAMCOMPLETED )
		{
			manta = 0;
			walrus = 0;
			
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA2) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA3) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA4) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_MANTA5) )
				manta++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS6) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS7) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS8) )
				walrus++;
			if( ContainAnnouncement(eMessage,ERADIO_UNIT_WALRUS9) )
				walrus++;

			if( manta > 1 || walrus > 1 )
			{
				CreateMultipleAnnouncement(eMessage,ERADIO_UNIT_PROGRAMCOMPLETED);

				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA2);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA3);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA4);
				CancelAnnouncement(eMessage,ERADIO_UNIT_MANTA5);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS6);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS7);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS8);
				CancelAnnouncement(eMessage,ERADIO_UNIT_WALRUS9);
			}
			return;
		}

		if ( eMessage == ERADIO_UNIT_LEAVINGTELEMETRY )
		{
			if ( eUnitMessage != ERADIO_NONE )
			{
				if( ContainAnnouncement(ERADIO_UNIT_OUTOFTELEMETRY,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_LEAVINGTELEMETRY,eUnitMessage);
			}
		}

		//Nerikat under attack kdyz se nahlasilo, ze utocim na nepritele
		//tak je jasne ze neco schytam...
		if ( eMessage == ERADIO_UNIT_UNDERATTACK )
		{
			if ( eUnitMessage != ERADIO_NONE )
			{
				if( ContainAnnouncement(ERADIO_UNIT_ENGAGEDENEMY,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_UNDERATTACK,eUnitMessage);
			}
		}

		//nehlasit ze je unit stuck furt dokola - dat tam aspon nejakou pauzu
		if ( eMessage == ERADIO_UNIT_ISSTUCK )
		{
			if ( eUnitMessage != ERADIO_NONE )
			{
				if( ContainAnnouncement(ERADIO_UNIT_ISSTUCK,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_ISSTUCK,eUnitMessage);
			}
		}
		
		//Nerikat ze mam malo benzinu, kdyz uz nemam zadnej
		if ( eMessage == ERADIO_UNIT_LOWFUEL )
		{
			if ( eUnitMessage != ERADIO_NONE )
			{
				if ( ContainAnnouncement(ERADIO_UNIT_NOFUEL,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_LOWFUEL,eUnitMessage);
			}
		}

		//Nerikat ze mam carrier hodne poskozenej, kdyz uz je mimo provoz
		if ( eMessage == ERADIO_CARRIER_HEAVILY_DAMAGED )
		{
			if ( eUnitMessage != ERADIO_NONE )
			{
				if ( ContainAnnouncement(ERADIO_CARRIER_INOPERABLE,eUnitMessage) )
					CancelAnnouncement(ERADIO_CARRIER_HEAVILY_DAMAGED,eUnitMessage);
			}
		}

		//Nerikat ze mam carrier poskozenej, kdyz se prehrava hlaska o velkem poskozeni
		if ( eMessage == ERADIO_CARRIER_TAKING_DAMAGE )
		{
			if ( eUnitMessage != ERADIO_NONE )
			{
				if ( ContainAnnouncement(ERADIO_CARRIER_HEAVILY_DAMAGED,eUnitMessage) )
					CancelAnnouncement(ERADIO_CARRIER_TAKING_DAMAGE,eUnitMessage);

				if ( ContainAnnouncement(ERADIO_CARRIER_INOPERABLE,eUnitMessage) )
					CancelAnnouncement(ERADIO_CARRIER_TAKING_DAMAGE,eUnitMessage);
			}
		}

		//zakazat damaged kdyz uz je znicena
		if ( eMessage == ERADIO_UNIT_HEAVILYDAMAGED )
		{
			if ( eUnitMessage != ERADIO_NONE )
			{
				if( ContainAnnouncement(ERADIO_UNIT_DESTROYED,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_HEAVILYDAMAGED,eUnitMessage);
			}
		}

		//pokud je jednotka znicena zakazat vsechny hlasky ktere se k ni vstahuji
		if ( eMessage == ERADIO_UNIT_DESTROYED )
		{
			if ( eUnitMessage != ERADIO_NONE )
			{
				if( ContainAnnouncement(ERADIO_UNIT_UNDERATTACK,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_UNDERATTACK,eUnitMessage);

				if( ContainAnnouncement(ERADIO_UNIT_ENGAGEDENEMY,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_ENGAGEDENEMY,eUnitMessage);

				if( ContainAnnouncement(ERADIO_UNIT_LOWFUEL,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_LOWFUEL,eUnitMessage);

				if( ContainAnnouncement(ERADIO_UNIT_LOWAMMO,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_LOWAMMO,eUnitMessage);

				if( ContainAnnouncement(ERADIO_UNIT_NOFUEL,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_NOFUEL,eUnitMessage);

				if( ContainAnnouncement(ERADIO_UNIT_NOAMMO,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_NOAMMO,eUnitMessage);

				if( ContainAnnouncement(ERADIO_UNIT_DOCKED,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_DOCKED,eUnitMessage);

				if( ContainAnnouncement(ERADIO_UNIT_LAUNCHED,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_LAUNCHED,eUnitMessage);

				if( ContainAnnouncement(ERADIO_UNIT_LEAVINGTELEMETRY,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_LEAVINGTELEMETRY,eUnitMessage);

				if( ContainAnnouncement(ERADIO_UNIT_LEAVINGTELEMETRY,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_LEAVINGTELEMETRY,eUnitMessage);

				if( ContainAnnouncement(ERADIO_UNIT_OUTOFTELEMETRY,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_OUTOFTELEMETRY,eUnitMessage);

				if( ContainAnnouncement(ERADIO_UNIT_BACKTOTELEMETRY,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_BACKTOTELEMETRY,eUnitMessage);

				if( ContainAnnouncement(ERADIO_UNIT_PROGRAMCOMPLETED,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_PROGRAMCOMPLETED,eUnitMessage);

				if( ContainAnnouncement(ERADIO_UNIT_READY,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_READY,eUnitMessage);

				if( ContainAnnouncement(ERADIO_UNIT_ONMOVE,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_ONMOVE,eUnitMessage);

				if( ContainAnnouncement(ERADIO_UNIT_ISSTUCK,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_ISSTUCK,eUnitMessage);

				if( ContainAnnouncement(ERADIO_UNIT_ISFUBAR,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_ISFUBAR,eUnitMessage);

				if( ContainAnnouncement(ERADIO_UNIT_HEAVILYDAMAGED,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_HEAVILYDAMAGED,eUnitMessage);

				if( ContainAnnouncement(ERADIO_UNIT_CANTLANDDUECARGO,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_CANTLANDDUECARGO,eUnitMessage);

				if( ContainAnnouncement(ERADIO_UNIT_CANTATTACK,eUnitMessage) )
					CancelAnnouncement(ERADIO_UNIT_CANTATTACK,eUnitMessage);
			}
		}
	}

	// assembly message for RepairSystem
	void OnAnnounceRepairMessage( string strAction, int iColor, int eRepairZone )
	{
		string msg;
		
		switch(eRepairZone)
		{
			case ERADIO_CARRIER_SUPERSTRUCTURE :
				msg = "#radio_zone_superstructure";
			break;
			case ERADIO_CARRIER_ENGINES :
				msg = "#radio_zone_engines";
			break;
			case ERADIO_CARRIER_CONTROLTOWER :
				msg = "#radio_zone_controltower";
			break;
			case ERADIO_CARRIER_OFFENSE :
				msg = "#radio_zone_offense";
			break;
			case ERADIO_CARRIER_DEFENSE :
				msg = "#radio_zone_defense";
			break;
			case ERADIO_CARRIER_CRUISEMISSILE :
				msg = "#radio_zone_cruisemissile";
			break;
			case ERADIO_CARRIER_MANTALIFTS :
				msg = "#radio_zone_mantalifts";
			break;
			case ERADIO_CARRIER_WALRUSLEFT :
				msg = "#radio_zone_walrusleft";
			break;
			case ERADIO_CARRIER_WALRUSRIGHT :
				msg = "#radio_zone_walrusright";
			break;
			case ERADIO_CARRIER_REPAIRSYSTEM :
				msg = "#radio_zone_repairsystem";
			break;
		}
		
		msg += " ";
		msg += strAction;
		g_Game.AddMessage( msg, iColor, LMF_EMPIRE );
	}

	// assembly message for unit
	void OnAnnounceMessageUnit( string strAction, int iColor, int eUnit )
	{
		string msg;
		
		switch(eUnit)
		{
			case ERADIO_UNIT_CARRIER :
				msg = "#radio_carrier";
			break;
			case ERADIO_UNIT_MANTA :
				msg = "#radio_manta";
			break;
			case ERADIO_UNIT_WALRUS :
				msg = "#radio_walrus";
			break;
			case ERADIO_UNIT_MANTA2 :
				msg = "#radio_manta2";
			break;
			case ERADIO_UNIT_MANTA3 :
				msg = "#radio_manta3";
			break;
			case ERADIO_UNIT_MANTA4 :
				msg = "#radio_manta4";
			break;
			case ERADIO_UNIT_MANTA5 :
				msg = "#radio_manta5";
			break;
			case ERADIO_UNIT_WALRUS6 :
				msg = "#radio_walrus6";
			break;
			case ERADIO_UNIT_WALRUS7 :
				msg = "#radio_walrus7";
			break;
			case ERADIO_UNIT_WALRUS8 :
				msg = "#radio_walrus8";
			break;
			case ERADIO_UNIT_WALRUS9 :
				msg = "#radio_walrus9";
			break;
		}

		msg += " ";
		msg += strAction;
		g_Game.AddMessage( msg, iColor, LMF_COMBAT );
	}


	//! Announce Single message
	void OnAnnounceSingle( int eMessage )
	{
		switch(eMessage)
		{
			// carrier
			case ERADIO_CARRIER_UAV :
				g_Game.AddMessage( "#radio_single_uavlaunched", 0xffffff00, LMF_COMBAT );
			break;
			case ERADIO_CARRIER_BARQUE :
				g_Game.AddMessage( "#radio_single_barque", 0xffffff00, LMF_EMPIRE );
			break;
			case ERADIO_CARRIER_SAILING :
				g_Game.AddMessage( "#radio_carrier_sailing", 0xffffff00, LMF_EMPIRE );
			break;

			case ERADIO_STRAT_DEPLOYDEVICEDESTROYED :
				g_Game.AddMessage( "#game_deploydevicedestroyed_01", COLOR_RED, LMF_COMBAT );
			break;

			case ERADIO_STRAT_CARRIERDEPLOYING :
				g_Game.AddMessage( "#game_carrierdeployingunits_01", COLOR_RED, LMF_COMBAT );
			break;

			//actor-unit messages
			case ERADIO_STRAT_CANTLEAVEVEHICLE_NOSPACE :
				g_Game.AddMessage( "#game_cantleavevehiclespace_01", COLOR_RED, LMF_COMBAT );
			break;

			case ERADIO_STRAT_CANTLEAVEVEHICLE_WATER :
				g_Game.AddMessage( "#game_cantleavevehiclewater_01", COLOR_RED, LMF_COMBAT );
			break;

			case ERADIO_STRAT_CANTLEAVEVEHICLE_SPEED :
				g_Game.AddMessage( "#game_cantleavevehiclespeed_01", COLOR_RED, LMF_COMBAT );
			break;
			

			// island messages
			case ERADIO_STRAT_COURSE_SET :
				g_Game.AddMessage( "#radio_single_courseset", COLOR_GREEN, LMF_EMPIRE );
			break;
			case ERADIO_STRAT_STOCKPILESELECTED :
				g_Game.AddMessage( "#radio_single_stockpileselected", COLOR_GREEN, LMF_EMPIRE );
			break;
			case ERADIO_STRAT_DESTINATION_REACHED :
				g_Game.AddMessage( "#radio_single_destinationreached", COLOR_GREEN, LMF_EMPIRE );
			break;

			// tactical messages
			case ERADIO_TACTICS_ASSIST :
				g_Game.AddMessage( "#radio_single_assist", COLOR_YELLOW, LMF_COMBAT );
			break;
			case ERADIO_TACTICS_ATTACK :
				g_Game.AddMessage( "#radio_single_attack", COLOR_YELLOW, LMF_COMBAT );
			break;
			case ERADIO_TACTICS_DEPLOY_ALL :
				g_Game.AddMessage( "#radio_single_undockall", COLOR_YELLOW, LMF_COMBAT );
			break;
			case ERADIO_TACTICS_DOCK_ALL :
				g_Game.AddMessage( "#radio_single_dockall", COLOR_YELLOW, LMF_COMBAT );
			break;
			case ERADIO_TACTICS_EMERGENCY :
				g_Game.AddMessage( "#radio_single_emergency", COLOR_YELLOW, LMF_COMBAT );
			break;
			case ERADIO_TACTICS_MOVE :
				g_Game.AddMessage( "#radio_single_supplyarrived", COLOR_YELLOW, LMF_COMBAT );
			break;
			case ERADIO_TACTICS_OFFENSIVE :
				g_Game.AddMessage( "#radio_single_offensive", COLOR_YELLOW, LMF_COMBAT );
			break;
			case ERADIO_TACTICS_DEFENSIVE :
				g_Game.AddMessage( "#radio_single_defensive", COLOR_YELLOW, LMF_COMBAT );
			break;
			case ERADIO_TACTICS_HOLD :
				g_Game.AddMessage( "#radio_single_hold", COLOR_YELLOW, LMF_COMBAT );
			break;
			case ERADIO_TACTICS_CONTINUE :
				g_Game.AddMessage( "#radio_single_continue", COLOR_YELLOW, LMF_COMBAT );
			break;
		}
	}

	//! Announce RepairSystem message
	void OnAnnounceRepairSystem( int eMessage, int eRepairZone )
	{
		switch(eMessage)
		{
			// carrier repair system
			case ERADIO_CARRIER_INOPERABLE :
				OnAnnounceRepairMessage( "#radio_carrier_inoperable", COLOR_YELLOW, eRepairZone );
			break;
			case ERADIO_CARRIER_TAKING_DAMAGE :
				OnAnnounceRepairMessage( "#radio_carrier_taking_damage", COLOR_RED, eRepairZone );
			break;
			case ERADIO_CARRIER_HEAVILY_DAMAGED :
				OnAnnounceRepairMessage( "#radio_carrier_heavily_damaged", COLOR_RED, eRepairZone );
			break;
			case ERADIO_CARRIER_REPAIRED :
				OnAnnounceRepairMessage( "#radio_carrier_reparied", COLOR_GREEN, eRepairZone );
			break;
			case ERADIO_CARRIER_REPAIRING :
				OnAnnounceRepairMessage( "#radio_carrier_repairing", COLOR_YELLOW, eRepairZone );
			break;
		}
	}

	//! Announce Unit message
	void OnAnnounceUnit( int eMessage, int eUnit )
	{
		switch(eMessage)
		{
			// unit messages
			case ERADIO_UNIT_UNDERATTACK :
				OnAnnounceMessageUnit( "#radio_unit_underattack", COLOR_RED, eUnit )
			break;
			case ERADIO_UNIT_ENGAGEDENEMY :
				OnAnnounceMessageUnit( "#radio_unit_engagedenemy", COLOR_YELLOW, eUnit )
			break;
			case ERADIO_UNIT_DESTROYED :
				OnAnnounceMessageUnit( "#radio_unit_destroyed", COLOR_RED, eUnit )
			break;
			case ERADIO_UNIT_LOWFUEL :
				OnAnnounceMessageUnit( "#radio_unit_lowfuel", COLOR_YELLOW, eUnit )
			break;
			case ERADIO_UNIT_LOWAMMO :
				OnAnnounceMessageUnit( "#radio_unit_lowammo", COLOR_YELLOW, eUnit )
			break;
			case ERADIO_UNIT_NOFUEL :
				OnAnnounceMessageUnit( "#radio_unit_nofuel", COLOR_RED, eUnit )
			break;
			case ERADIO_UNIT_NOAMMO :
				OnAnnounceMessageUnit( "#radio_unit_noammo", COLOR_RED, eUnit )
			break;
			case ERADIO_UNIT_DOCKED :
				OnAnnounceMessageUnit( "#radio_unit_docked", COLOR_GREEN, eUnit )
			break;
			case ERADIO_UNIT_LAUNCHED :
				OnAnnounceMessageUnit( "#radio_unit_launched", COLOR_GREEN, eUnit )
			break;
			case ERADIO_UNIT_LEAVINGTELEMETRY :
				OnAnnounceMessageUnit( "#radio_unit_leavingtelemetry", COLOR_YELLOW, eUnit )
			break;
			case ERADIO_UNIT_OUTOFTELEMETRY :
				OnAnnounceMessageUnit( "#radio_unit_outoftelemetry", COLOR_YELLOW, eUnit )
			break;
			case ERADIO_UNIT_BACKTOTELEMETRY :
				OnAnnounceMessageUnit( "#radio_unit_backtotelemetry", COLOR_YELLOW, eUnit )
			break;
			case ERADIO_UNIT_PROGRAMCOMPLETED :
				OnAnnounceMessageUnit( "#radio_unit_programcompleted", COLOR_GREEN, eUnit )
			break;
			case ERADIO_UNIT_READY :
				OnAnnounceMessageUnit( "#radio_unit_ready", COLOR_GREEN, eUnit )
			break;
			case ERADIO_UNIT_ONMOVE :
				OnAnnounceMessageUnit( "#radio_unit_onmove", COLOR_GREEN, eUnit )
			break;
			case ERADIO_UNIT_ISSTUCK :
				OnAnnounceMessageUnit( "#radio_unit_isstuck", COLOR_RED, eUnit )
			break;
			case ERADIO_UNIT_ISFUBAR :
				OnAnnounceMessageUnit( "#radio_unit_fubar", COLOR_RED, eUnit )
			break;
			case ERADIO_UNIT_HEAVILYDAMAGED :
				OnAnnounceMessageUnit( "#radio_unit_heavilydamaged", COLOR_RED, eUnit )
			break;
			case ERADIO_UNIT_CANTLANDDUECARGO :
				OnAnnounceMessageUnit( "#radio_unit_cantlandduecargo", COLOR_RED, eUnit )
			break;
			case ERADIO_UNIT_CANTATTACK :
				OnAnnounceMessageUnit( "#radio_unit_cantattack", COLOR_RED, eUnit )
			break;
		}
	}

	//! Announce message for multiple Units
	void OnAnnounceMultipleUnits( int eMessage, string sUnitList )
	{
		string msg;

		msg = "#radio_units";
		msg += " ";
		msg += sUnitList;
		msg += " ";
		switch(eMessage)
		{
			// unit messages
			case ERADIO_UNIT_UNDERATTACK :
				msg += "#radio_unit_underattack"
				g_Game.AddMessage( msg, COLOR_YELLOW, LMF_COMBAT );
			break;
			case ERADIO_UNIT_ENGAGEDENEMY :
				msg += "#radio_unit_engagedenemy";
				g_Game.AddMessage( msg, COLOR_YELLOW, LMF_COMBAT );
			break;
			case ERADIO_UNIT_DESTROYED :
				msg += "#radio_unit_destroyed";
				g_Game.AddMessage( msg, COLOR_RED, LMF_COMBAT );
			break;
			case ERADIO_UNIT_DOCKED :
				msg += "#radio_unit_docked";
				g_Game.AddMessage( msg, COLOR_GREEN, LMF_COMBAT );
			break;
			case ERADIO_UNIT_LAUNCHED :
				msg += "#radio_unit_launched";
				g_Game.AddMessage( msg, COLOR_GREEN, LMF_COMBAT );
			break;
			case ERADIO_UNIT_LEAVINGTELEMETRY :
				msg += "#radio_unit_leavingtelemetry";
				g_Game.AddMessage( msg, COLOR_YELLOW, LMF_COMBAT );
			break;
			case ERADIO_UNIT_OUTOFTELEMETRY :
				msg += "#radio_unit_outoftelemetry";
				g_Game.AddMessage( msg, COLOR_YELLOW, LMF_COMBAT );
			break;
			case ERADIO_UNIT_BACKTOTELEMETRY :
				msg += "#radio_unit_backtotelemetry";
				g_Game.AddMessage( msg, COLOR_YELLOW, LMF_COMBAT );
			break;
			case ERADIO_UNIT_PROGRAMCOMPLETED :
				msg += "#radio_unit_programcompleted";
				g_Game.AddMessage( msg, COLOR_GREEN, LMF_COMBAT );
			break;
			case ERADIO_UNIT_READY :
				msg += "#radio_unit_ready";
				g_Game.AddMessage( msg, COLOR_GREEN, LMF_COMBAT );
			break;
			case ERADIO_UNIT_ONMOVE :
				msg += "#radio_unit_onmove";
				g_Game.AddMessage( msg, COLOR_GREEN, LMF_COMBAT );
			break;
			case ERADIO_UNIT_ISFUBAR :
				msg += "#radio_unit_fubar";
				g_Game.AddMessage( msg, COLOR_RED, LMF_COMBAT );
			break;
			case ERADIO_UNIT_HEAVILYDAMAGED :
				msg += "#radio_unit_heavilydamaged";
				g_Game.AddMessage( msg, COLOR_RED, LMF_COMBAT );
			break;
			case ERADIO_UNIT_CANTLANDDUECARGO :
				msg += "#radio_unit_cantlandduecargo";
				g_Game.AddMessage( msg, COLOR_RED, LMF_COMBAT );
			break;
			case ERADIO_UNIT_CANTATTACK :
				msg += "#radio_unit_cantattack";
				g_Game.AddMessage( msg, COLOR_RED, LMF_COMBAT );
			break;
		}
		//msg += " doing action or something similar";

		//g_Game.AddMessage( msg, COLOR_YELLOW, LMF_COMBAT );
	}

	//! Announce Island message
	void OnAnnounceIsland( int eMessage, int iIsland )
	{
	}

};